      subroutine dtdx2(nn,parm,x,x0,ptype,nmoho,iulst,tmin,dx,delta,
     &aninc,az0,iflag)

C***********************************************************************
C
c   subroutine to calculate travel time and derivatives for a one-
c   dimensional nl-layered model
c
c       Barry R. Lienert    March, 1991
C
c   Adapted for variable station elevations Jan, 1987
c   can now use x0(3) with any positive value i.e. station
c   can be positioned in any layer. However the top of the velocity
c   model must then correspond to the highest elevation station IN THE
c   WHOLE ARRAY.
c
c   changed to work with geocentric lat/long instead of km 3/14/91
c   Lat./Long. are now in geocentric radians on a sphere having the
c   earth's equatorial (maximum) radius (6378.2 km)
c
c   inputs:    nn:     number of parameters (=2*nl-1)
c              parm(i),i=1,nl:     layer velocities in km/sec
c              parm(i),i=nl+1,nn:  layer thicknesses in km
c              x:      event coordinates in geocentric radians
c                      (depth still in km)
c              x0:     station coordinates in geocentric radians
c                      (depth in km)
c                      - x0(3) is now depth below top of top layer
c              ptype:  G        : direct arrival
c                      N        : moho refracted arrival
c                      blank    : minimum arrival 
c              nmoho:  layer# having moho velocity
c              iulst:  output listing unit# for error messages
c                      (if <= 0, no output)
c
c   outputs    tmin:   minimum travel time
c              dx(1):  spatial derivative of tmin wrt long. in secs/radian
c              dx(2):  spatial derivative of tmin wrt lat. in secs/radian
c              dx(3)   spatial derivative of tmin wrt depth in secs/radian
c                      (the depth derivative in secs/km is multiplied by 6367,
c                       the earth's mean radius, to keep it similar in size 
c                       to dx(1) and dx(2))
c              delta:  epicentral distance in km
c              aninc:    angle (measured upward from vertical) at which
c                      ray leaves the source (< 90 <=> refracted)
c              iflag:  1 for valid phase, 0 otherwise
c
c   subroutines called:
c
c              del :calculates horizontal offset for direct path
c                   - modified version for variable station depth
c
C***********************************************************************

      character*1 ptype
c     integer iflag
      REAL x,x0,eqrad,eps,sthi,cthi,dx,eta2,tmin,sum,ss,parm,eta1,ea,eb
      REAL delta,dedeg,az0,cfact,dellat,dellon,dist,aninc,dd,kmin,dcrit
      REAL e,t,td,pm,et,thi
      INTEGER nl,nn,ne2,ne1,i,na,nb,nmoho,iflag,nep1,n1,n2,k,ne,ii,iulst
      dimension x(3),x0(3),dx(3),parm(50),dcrit(50)
      double precision t1,t2,u,u1,u2,dtan,datan,dsqrt,del,dl1,dl2,dl,
     +p,p1,p2,dabs,dble,pi,degtorad
      if(abs(x(3)-x0(3)).lt.0.001)x(3)=x0(3)
      pi=3.14159265d0
      degtorad=pi/180.d0
      eqrad=6378.2
      eps=0.01d0
      sthi=0.0
      cthi=1.0
      dx(1)=0.0
      dx(2)=0.0

c    nl=no of layers

      nl=(nn+1)/2
      ne2=1
      ne1=1
      eta2=x0(3)
      tmin=1.e22
      if(nl.eq.1)then
        ne1=1
        go to 59
      endif

c    find find layer, ne2, that station is in

      sum=0.0

      do 3 i=1,nl-1
        ss=parm(nl+i)
        sum=sum+ss
        if(x0(3).le.sum)go to 4
3     continue

      ss=0.0
      i=nl
4     ne2=i

c    eta2 is the depth to the station from the top of this layer

      eta2=x0(3)-sum+ss

c   now find layer, ne1, that event is in

      sum=0.0

      do 1 i=1,nl-1
        ss=parm(nl+i)
        sum=sum+ss
        if(x(3).le.sum)go to 2
1     continue

      ss=0.0
      i=nl
2     ne1=i

c    eta1 is the depth to the event from the top of this layer

      eta1=x(3)-sum+ss

c    if eta1 < 0.001, set ne1 to previous layer 

      if(eta1.lt.0.1.and.ne1.ne.1)then
         eta1=eta1+parm(nl+ne1)
         ne1=ne1-1
c        eta1=parm(nl+ne1)+eta1*parm(ne1+1)/parm(ne1)
      endif
      na=ne2
      ea=parm(nl+ne2)-eta2
      eb=eta1
      nb=ne1

c    station is below hypocenter

      if(ne2.gt.ne1)then
        nb=ne2
        ea=parm(nl+ne1)-eta1
        na=ne1
        eb=eta2
      endif

59    continue

c     use J. Lahr's routine to get delta from geocentric distances
c     az0 is the azimuth of the station from the epicenter
      call delaz(x0(2),x0(1),delta,dedeg,az0,x(2),x(1))
      delta=abs(delta)
      dedeg=abs(dedeg)

c    cfact is local conversion factor between km and radians in rad/km

      cfact=sngl(degtorad)*dedeg/delta
      dellat=delta*cos(az0*sngl(degtorad))
      dellon=cos(abs(x(2)))*delta*sin(az0*sngl(degtorad))
      dist=sqrt(delta**2+(x(3)-x0(3))**2)

c*************************************************************************
c                       trivial cases

c  1.  ptype='N' and no refractions

      if(ptype.eq.'N'.and.ne1.ge.nmoho)then
        iflag=0
        aninc=0.0
        return
      endif

      if(nl.eq.1.and.ptype.eq.'N')then
        iflag=0
        aninc=0.0
        return
      endif

      if(dist.lt.0.001)then

c  2. event coincident with station

        do 17 i=1,2
17      dx(i)=-1.0/(cfact*parm(ne1))

        dx(1)=dx(1)*cos(abs(x(2)))
        dx(3)=eqrad/parm(ne1)

c  incidence angle set to zero (???) for this case

        aninc=0
        iflag=1
        if(ptype.eq.'N')iflag=0
        tmin=0.0
        aninc=0.0
        return
      endif

      if(delta.lt.0.001)go to 24


c  3.  single layer case: only direct wave possible

      if(nl.eq.1)then
        tmin=dist/parm(1)
        dx(1)=-dellon/(cfact*dist*parm(1))
        dx(2)=-dellat/(cfact*dist*parm(1))
        dx(3)=(x(3)-x0(3))*eqrad/(dist*parm(1))
        if(x(3).ne.x0(3))then
          dd=atan(delta/(x(3)-x0(3)))/sngl(degtorad)
          if(dd.ge.0.0)then
            aninc=180.0-dd
          else
            aninc=abs(dd)
          endif
        else
          aninc=90.0
        endif
        iflag=1
        return
      endif

      if(ptype.eq.'G')go to 66

c  4.  station or event in bottom layer - no refractions

      if(ne1.eq.nl.or.ne2.eq.nl)then
        if(ptype.eq.'N')then
          iflag=0
          aninc=0.0
          return
        else
          go to 66
        endif
      endif
c*******************************************************************
    
c   nep1 is the shallowest layer in which refraction can occur

      nep1=ne1+1
      if(ne2+1.gt.nep1)nep1=ne2+1
      if(nep1.gt.nl)then
        if(ptype.eq.'N')then
          iflag=0
          aninc=0.0
          return
        else
          go to 66
        endif
      endif
      if(delta.eq.0.0)go to 24
      if(ne1.eq.nl.or.ne2.eq.nl)then
        if(ptype.eq.'N')then
          iflag=0
          aninc=0.0
          return
        else
          go to 66
        endif
      endif

c  find the refracted wave travel times

c  kmin is the fastest refraction layer, tmin is its travel time

      kmin=0

c   find critical distances, dcrit(k), for all refracted waves. if no
c   refracted wave is possible, set dcrit(k)=-1.0

      n1=ne2

c  need this section for when station is below hypocenter

      if(ne2.gt.ne1)n1=ne1
      n2=ne1-1
      if(ne2.gt.ne1)n2=ne2-1

      do 50 k=nep1,nl
        sum=0.0
        do 40 i=nep1-1,k-1
          if(parm(k).le.parm(i))go to 49
40      sum=sum+parm(nl+i)*2./sqrt((parm(k)/parm(i))**2-1.)

        if(parm(k).le.parm(ne1).or.parm(k).le.parm(ne2))go to 49
        dcrit(k)=sum-eta1/sqrt((parm(k)/parm(ne1))**2-1.)
     &  -eta2/sqrt((parm(k)/parm(ne2))**2-1.)

        if(ne1.ne.ne2)then
          do 42 i=n1,n2
            if(parm(k).le.parm(i))go to 49
42        dcrit(k)=dcrit(k)+parm(nl+i)/sqrt((parm(k)/parm(i))**2-1.)
        endif

        if(dcrit(k).le.delta)go to 50
49      dcrit(k)=-1.0
      
50    continue
    

c   find the travel times, t, for all possible refracted waves

      do 60 k=nep1,nl
        if(dcrit(k).lt.0.0)go to 60
        sum=0.0

        do 52 i=nep1-1,k-1
          e=parm(nl+i)*2.
52      sum=sum+e*sqrt(1./(parm(i)*parm(i))-1./(parm(k)*parm(k)))

        if(parm(ne1).gt.parm(k))go to 60
        t=delta/parm(k)-eta1*sqrt(1./(parm(ne1)*parm(ne1))
     &  -1./(parm(k)*parm(k)))+sum-eta2*sqrt(1./(parm(ne2)*parm(ne2))
     &  -1./(parm(k)*parm(k)))

        if(ne1.ne.ne2)then
          do 53 i=n1,n2
53        t=t+parm(nl+i)*sqrt(1./(parm(i)*parm(i))-1./1/(parm(k)*
     &    parm(k)))
        endif

c   if t < tmin, set tmin = t

        if(t.lt.tmin)then
          kmin=k
          tmin=t
        endif

c    N phase: if k = nmoho  then exit

        if(ptype.eq.'N'.and.nmoho.eq.k)then
          kmin=k
          tmin=t
          go to 62
        endif

60    continue

c     exit if ptype='N' and  no refracted wave is possible

      if(ptype.eq.'N'.and.kmin.eq.0)then
        iflag=0
        aninc=0.0
        return
      endif

66    continue

c    Direct travel times


c    Trivial cases: (i) same depth

      if(abs(x(3)-x0(3)).lt.0.001)go to 36

c                   (ii) station and event in same layer

      if(ne1.eq.ne2)then
        td=dist/parm(ne1)
        if(td.lt.tmin.or.ptype.eq.'G')then
          tmin=td
          dx(1)=-dellon/(cfact*dist*parm(ne1))
          dx(2)=-dellat/(cfact*dist*parm(ne1))
          dx(3)=(x(3)-x0(3))*eqrad/(dist*parm(ne1))
          if(x(3).ne.x0(3))then
            dd=atan(delta/(x(3)-x0(3)))/sngl(degtorad)
            if(dd.ge.0.0)then
              aninc=180.-dd
            else
              aninc=abs(dd)
            endif
          else
            aninc=90.0
          endif
          iflag=1
          return
        endif
      endif

c   this section calculates an approximate direct path travel time (a bit
c   less than the true value) to compare with the refracted time, tmin
c   since iterative calculation of the exact direct path time is relatively
c   time consuming  and in many applications is never used. if this time
c   is less than tmin, the exact iterative direct path time is calculated
c   the saving in calculation time can be quite significant

      td=0.0

      if(kmin.ne.0.and.ptype.ne.'G')then
        k=kmin
        t1=dble(abs(x(3)-x0(3)))/dble(delta)
        p=1.d0/(dble(parm(ne1))*dsqrt(1.d0+t1*t1))
  
        do 61 i=na,nb
          pm=parm(nl+i)
          if(i.eq.na)pm=ea
          if(i.eq.nb)pm=eb
61      td=td+sngl(dble(pm)/(dble(parm(i))*
     &  dsqrt(1.d0-p*p*dble(parm(i))**2)))

      endif

      if(td.gt.tmin.and.ptype.ne.'G')go to 62

c   calculate the direct path travel time, td, accurately

c    find tangents of maximum and minimum takeoff angles,t1 and t2

      et=eta1
      ne=ne1
      if(ne2.gt.ne1)et=eta2
      if(ne2.gt.ne1)ne=ne2
      if(ne1.ne.ne2)go to 39
      if(x(3).eq.x0(3))go to 36
      td=dist/parm(ne1)
      thi=atan(delta/abs(x(3)-x0(3)))
      cthi=cos(thi)
      sthi=sin(thi)
      go to 37
39    continue
      t1=dble(delta)/dble(abs(x(3)-x0(3)))
      t2=dble(delta)/dble(et)
      u1=dble(et)*t1
      u2=dble(delta)

c   find raypath parameters, p1 and p2 for direct waves

      p1=1.d0/(dsqrt(1.d0+1.d0/(t1*t1))*dble(parm(ne)))
      p2=1.d0/(dsqrt(1.d0+1.d0/(t2*t2))*dble(parm(ne)))
      p=p1
      sthi=sngl(p1)*parm(ne)
      cthi=sqrt(1.-sthi*sthi)

c   find corresponding distances, dl1 and dl2 for the 2 rays

      dl1=del(p1,eta1,ne1,eta2,ne2,nn,parm)
      if(dabs(dl1-dble(delta)).le.eps)go to 30
41    if(dl1.le.dble(delta))go to 43
      t1=dtan(2.d0*datan(t1)-datan(dl1/dble(abs(x(3)-x0(3)))))
      p1=1.d0/(dsqrt(1.d0+1.d0/(t1*t1))*dble(parm(ne)))
      dl1=del(p1,eta1,ne1,eta2,ne2,nn,parm)
      go to 41
43    continue
      dl2=del(p2,eta1,ne1,eta2,ne2,nn,parm)
      if(dl1.eq.dl2)go to 30
      u1=dble(et)*t1
      u2=dble(delta)

c   now find takeoff angle thi and distance delta1 of a ray p which
c   lies between p1 and p2

      ii=0
10    u=u1+(u2-u1)*(dble(delta)-dl1)/(dl2-dl1)
      ii=ii+1
      p=u/(dble(parm(ne))*dsqrt(u*u+dble(et)*dble(et)))
      sthi=sngl(p)*parm(ne)
      cthi=sqrt(1.0-sthi*sthi)
      dl=del(p,eta1,ne1,eta2,ne2,nn,parm)

c   iterate until abs(delta-dl)<eps

      if(dabs(dl-dble(delta)).le.eps)go to 30
      if(dabs(dl1-dble(delta)).gt.eps)go to 770
      dl=dl1
      u=u1
      go to 30
770   if(dabs(dl2-dble(delta)).gt.eps)go to 771
      dl=dl2
      u=u2
      go to 30
771   if(ii.gt.50)go to 26
      if(dl.gt.dble(delta))go to 20
      dl1=dl
      u1=u
      go to 10
20    dl2=dl
      u2=u
      go to 10
26    continue
      if(iulst.gt.0)then
        write(iulst,100)
        write(iulst,101)delta,dl
100     format(' no direct path convergence after 50 iterations')
101     format(' delta = ',f12.4,' dl = ',f12.4)
      endif
      iflag=0
      aninc=0.0
      return

24    p=0.0d0
30    continue

c   calculate the travel time,td,for the direct path

      td=0.0

      do 35 i=na,nb
        pm=parm(nl+i)
        if(i.eq.na)pm=ea
        if(i.eq.nb)pm=eb
        if(parm(i)*sngl(p).ge.1.0)go to 38
35    td=td+pm/(parm(i)*sqrt(1.-p*p*parm(i)*parm(i)))

      go to 37
38    continue
      if(iulst.gt.0)then
        write(iulst,'('' p > 1/velocity in layer '',
     &  i3,'' p = '',f6.4)')i,p
      endif
      iflag=0
      aninc=0.0
      return

36    td=delta/parm(ne1)
      cthi=0.0
      sthi=1.0

37    if(td.gt.tmin.and.ptype.ne.'G')go to 62

c    set minimum travel time to td

      tmin=td
      
c   direct path derivatives

      dx(1)=0.0
      dx(2)=0.0
      if(delta.lt.0.001)go to 69
      dx(1)=-dellon*sthi/(cfact*delta*parm(ne1))
      dx(2)=-dellat*sthi/(cfact*delta*parm(ne1))
69    dx(3)=cthi*eqrad/parm(ne1)
      if(x0(3).gt.x(3))dx(3)=-dx(3)
      if(cthi.ne.0.0)then
        aninc=180.-abs(atan(sthi/cthi))/sngl(degtorad)
      else
        aninc=90.0
      endif
      iflag=1
      return
 

c   refracted path is minimum - find spatial derivatives of tmin and exit
 
62    k=kmin
      dx(1)=-dellon/(cfact*delta*parm(k))
      dx(2)=-dellat/(cfact*delta*parm(k))
      if(parm(k).lt.parm(ne1))then
        if(iulst.gt.0)then
          write(iulst,'('' *** Illegal velocity inversion ***'')')
          write(iulst,500)ne1,kmin,x(3),parm(ne1),parm(k)
        endif
500     format(2i8,3f12.4)
        aninc=0.0
        iflag=0
        return
      endif
      dx(3)=-eqrad*sqrt(1./(parm(ne1)*parm(ne1))-1./(parm(k)
     &*parm(k)))
      dd=sqrt(parm(k)*parm(k)-parm(ne1)*parm(ne1))
      if(dd.ne.0.0)then
        aninc=atan(parm(ne1)/dd)/sngl(degtorad)
      else
        aninc=90.0
      endif
      iflag=1
      return

      end
